/*
 * Copyright 2011 Andres Gomez Casanova
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.mgaip;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.Writer;

import org.jdom.Comment;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.jdom.xpath.XPath;

/**
 * Class that replaces the already existent reports.
 * <p>
 * <b>Control Version</b>
 * <p>
 * <ul>
 * <li>1.0 Class creation.</li>
 * </ul>
 *
 * @author Jose Francisco "Pachito" Saray
 * @author Andres Gomez Casanova <a
 *         href="mailto:a n g o c a at y a h o o dot c o m">(AngocA)</a>
 * @version 1.0 20110529
 * @since 1.0
 */
public final class Replacer {

    /**
     * Internal id for horizontal ad.
     */
    private static final int HORIZONTAL_AD = 2;
    /**
     * Internal id for logo ad.
     */
    private static final int LOGO_AD = 4;
    /**
     * Internal id for unique ad.
     */
    private static final int UNIQUE_AD = 3;
    /**
     * Internal id for vertical ad.
     */
    private static final int VERTICAL_AD = 1;

    /**
     * Mojo that calls the replacer.
     */
    private AdInserterMojo report;

    /**
     * Instantiates the mojo.
     *
     * @param caller
     *            Mojo containing all parameters.
     */
    Replacer(final AdInserterMojo/* ! */caller) {
        this.report = caller;
    }

    /**
     * Replaces an HTML tag with the given type of ad.
     *
     * @param single
     *            XML component that contains the location of the tag to
     *            replace.
     * @param content
     *            Content to insert.
     * @return true because the HTML has been modified and it has to be
     *         rewritten.
     */
    private boolean changeHTMLTag(final Element/* ! */single,
            final String/* ! */content) {
        Element parelement = single.getParentElement();
        single.detach();
        single.removeChild("img");
        Element script = new Element("script");
        script.setAttribute("type", "text/javascript");
        Comment comment = new Comment(content);
        script.addContent(comment);
        parelement.addContent(script);
        Element common = this.getCommon();
        parelement.addContent(common);
        return true;
    }

    /**
     * Returns the Google ad according to the requested type.
     *
     * @param type
     *            Type of ad.
     * @return Script to show this ad in html.
     */
    private String/* ! */getAdContent(final int type) {
        // assert type >= 1 && type <=4 : "Unknown ad type";

        String ret = "\n";
        ret += "google_ad_client = \"" + report.googleId + "\";\n";
        switch (type) {
        case HORIZONTAL_AD:
            ret += "google_ad_slot = \"" + report.horizontalAdId + "\";\n";
            ret += "google_ad_width = " + report.horizontalAdWidth + ";\n";
            ret += "google_ad_height = " + report.horizontalAdHeight + ";\n";
            break;
        case LOGO_AD:
            ret += "google_ad_slot = \"" + report.logoAdId + "\";\n";
            ret += "google_ad_width = " + report.logoAdWidth + ";\n";
            ret += "google_ad_height = " + report.logoAdHeight + ";\n";
            break;
        case UNIQUE_AD:
            ret += "google_ad_slot = \"" + report.uniqueAdId + "\";\n";
            ret += "google_ad_width = " + report.uniqueAdWidth + ";\n";
            ret += "google_ad_height = " + report.uniqueAdHeight + ";\n";
            break;
        case VERTICAL_AD:
            ret += "google_ad_slot = \"" + report.verticalAdId + "\";\n";
            ret += "google_ad_width = " + report.verticalAdWidth + ";\n";
            ret += "google_ad_height = " + report.verticalAdHeight + ";\n";
            break;
        default:
            this.report.getLog().info("No content assigned.");
            break;
        }
        ret += "//";

        return ret;
    }

    /**
     * @return Returns the common part of Google Ad.
     */
    private Element/* ! */getCommon() {
        Element common = new Element("script");
        common.setAttribute("type", "text/javascript");
        common.setAttribute("src",
                "http://pagead2.googlesyndication.com/pagead/show_ads.js");
        common.setText("mgaip");
        return common;
    }

    /**
     * Replaces the customPoweredBy tag with the given code.
     * <p>
     * TODO this should call an external javascript file that will contain the
     * code and stuff to show here.
     *
     * @param single
     *            Element where the custom code has to be inserted.
     * @return true to indicate that the file has been modified..
     */
    private boolean/* ! */getCustomPoweredBy(final Element/* ! */single) {
        Element parelement = single.getParentElement();
        single.detach();
        single.removeChild("img");
        Element custom = new Element("div");
        // TODO the next line should be inserted as raw, not as text.
        custom.addContent(report.customPoweredBy);
        parelement.addContent(custom);
        return true;
    }

    /**
     * Returns the I like Facebook script.
     * <p>
     * <code>
     * &lt;iframe src=
     * "http://www.facebook.com/plugins/like.php?
     * href=https%3A%2F%2Fwww.facebook.com%2Fpages%2FZemucan%2F138284808329
     * &amp;send=false&amp;layout=box_count&amp;width=150&amp;show_faces=false
     * &amp;action=like&amp;colorscheme=light&amp;font&amp;height=90"
     * scrolling="no" frameborder="0"
     * style="border:none; overflow:hidden; width:150px; height:90px;"
     * allowTransparency="true"&gt;&lt;/iframe&gt;
     * </code>
     * <p>
     * The example is for Zemucan, but it is defined in the variable
     * facebookLink. There are two types of links:
     * <ul>
     * <li>
     * In pages, with number:
     * https://www.facebook.com/pages/IBM-InfoSphere-Warehouse/17628877758</li>
     * <li>
     * Direct: https://www.facebook.com/PHP</li>
     * </ul>
     * <p>
     * However the link has to be transformed from
     * <p>
     * <code>https://www.facebook.com/pages/Zemucan/138284808329
     * </code>
     * <p>
     * to
     * <p>
     * <code>https%3A%2F%2Fwww.facebook.com%2Fpages%2FZemucan%2F138284808329
     * </code>
     *
     * @param single
     *            Element where the code has to be inserted.
     * @return true to indicate that the file has been modified.
     */
    private boolean/* ! */getFacebookILike(final Element/* ! */single) {
        String src = "http://www.facebook.com/plugins/like.php";
        src += "?href=" + report.facebookLink + "&";
        src += "send=false&layout=box_count&width=150&";
        src += "show_faces=false&action=like&colorscheme=light&";
        src += "font&height=90";
        String style = "border:none; overflow:hidden; width:150px; "
                + "height:90px;";

        Element parelement = single.getParentElement();
        single.detach();
        single.removeChild("img");
        Element iframe = new Element("iframe");
        iframe.setAttribute("src", src);
        iframe.setAttribute("scrolling", "no");
        iframe.setAttribute("frameborder", "0");
        iframe.setAttribute("style", style);
        iframe.setAttribute("allowTransparency", "true");
        iframe.setText("mgaip");
        parelement.addContent(iframe);
        return true;
    }

    /**
     * Searches the tags to modify the HTML file and call the respective method
     * to insert an ad.
     * <p>
     * Vertical (xpath: html/body/div id="leftColumn"/div id="navcolumn"
     * title="googleAds"):
     * <p>
     * <code>
     * &lt;a href="" title="googleAds" class="poweredBy"&gt;<br>
     * &lt;logo name="googleAds" img="googleAds"/&gt;<br>
     * &lt;/a&gt;
     * </code>
     * <p>
     * Horizontal (xpath: html/body/div id="bodyColumn"/div id="contextBox"/div
     * class="section"):
     * <p>
     * <code>
     * &lt;img src="googleAds" alt="googleAds" /&gt;
     * </code>
     * <p>
     * Unique (xpath: html/body/div id="bodyColumn"/div id="contextBox"/div
     * class="section"/p):
     * <p>
     * <code>
     * &lt;a href="#googleAds"&gt;googleAds&lt;/a&gt;
     * </code>
     * <p>
     * Logo (xpath: html/body/div id="banner"/span id="bannerLeft" |
     * id="bannerRight"):
     * <p>
     * <code>
     * &lt;img src="googleAds" alt="googleAds" /&gt;
     * </code>
     * <p>
     * And then put the publicity:
     * <p>
     * <code>
     * &lt;script type="text/javascript"&gt;&lt;!--<br>
     * google_ad_client = "XXXXX";<br>
     * google_ad_slot = "YYYYY";<br>
     * google_ad_width = 120;<br>
     * google_ad_height = 600;<br>
     * //--&gt;<br>
     * &lt;/script&gt;<br>
     * &lt;script type="text/javascript"<br>
     * src="http://pagead2.googlesyndication.com/pagead/show_ads.js"&gt;<br>
     * &lt;/script&gt;
     * </code>
     * <p>
     * Where XXXXX the google user id, and YYYYY is the ad identifier.
     * <p>
     * Also, it replaces the powered by for facebook (xpath: html/body/div
     * id="leftColumn"/div id="navcolumn" title="facebookILike").
     * <p>
     * <code>
     * &lt;a href="" title="facebookILike" class="poweredBy"&gt;<br>
     * &lt;logo name="facebookILike" img="facebookILike"/&gt;<br>
     * &lt;/a&gt;
     * </code>
     * <p>
     * And a custom code as powered by (xpath: html/body/div id="leftColumn"/div
     * id="navcolumn" title="customPoweredBy").
     * <p>
     * <code>
     * &lt;a href="" title="customPoweredBy" class="poweredBy"&gt;<br>
     * &lt;logo name="customPoweredBy" img="customPoweredBy"/&gt;<br>
     * &lt;/a&gt;
     * </code>
     * <p>
     *
     * @param elem
     *            Root element.
     * @return true if the HTML has been modified and it has to be rewritten,
     *         false otherwise.
     * @throws JDOMException
     *             If there is a problem
     */
    private boolean htmlReplacement(final Element/* ! */elem)
            throws JDOMException {
        boolean modified = false;
        // FIXME when there are multiple powered by, there is a problem in the
        // order of insertion.
        // PoweredBy
        XPath xpath = XPath.newInstance("//div[@id='leftColumn']"
                + "/div[@id='navcolumn']/a[@class='poweredBy' "
                + "and @title='googleAds']");
        Element single = (Element) xpath.selectSingleNode(elem);
        if (single != null) {
            report.getLog().debug("Powered - Vertical");
            modified = this.changeHTMLTag(single,
                    this.getAdContent(VERTICAL_AD));
        }

        // Horizontal
        xpath = XPath.newInstance("//div[@id='bodyColumn']"
                + "/div[@id='contentBox']/div[@class='section']"
                + "/img[@src='googleAds' and @alt='googleAds']");
        single = (Element) xpath.selectSingleNode(elem);
        if (single != null) {
            report.getLog().debug("Horizontal");
            modified = this.changeHTMLTag(single,
                    this.getAdContent(HORIZONTAL_AD));
        }

        // Unique
        xpath = XPath.newInstance("//div[@id='bodyColumn']"
                + "/div[@id='contentBox']/div[@class='section']/p"
                + "/a[@href='#googleAds']");
        single = (Element) xpath.selectSingleNode(elem);
        if (single != null) {
            report.getLog().debug("Unique");
            modified = this.changeHTMLTag(single, this.getAdContent(UNIQUE_AD));
        }

        // Logo
        xpath = XPath.newInstance("//div[@id='banner']"
                + "/span[@id='bannerRight']"
                + "/img[@src='googleAds' and @alt='googleAds']");
        single = (Element) xpath.selectSingleNode(elem);
        if (single != null) {
            report.getLog().debug("Logo right");
            modified = this.changeHTMLTag(single, this.getAdContent(LOGO_AD));
        }
        xpath = XPath.newInstance("//div[@id='banner']"
                + "/span[@id='bannerLeft']"
                + "/img[@src='googleAds' and @alt='googleAds']");
        single = (Element) xpath.selectSingleNode(elem);
        if (single != null) {
            report.getLog().debug("Logo left");
            modified = this.changeHTMLTag(single, this.getAdContent(LOGO_AD));
        }

        // Facebook
        xpath = XPath.newInstance("//div[@id='leftColumn']"
                + "/div[@id='navcolumn']/a[@class='poweredBy' "
                + "and @title='facebookILike']");
        single = (Element) xpath.selectSingleNode(elem);
        if (single != null) {
            report.getLog().debug("Powered - Facebook");
            modified = this.getFacebookILike(single);
        }

        // Custom
        xpath = XPath.newInstance("//div[@id='leftColumn']"
                + "/div[@id='navcolumn']/a[@class='poweredBy' "
                + "and @title='customPoweredBy']");
        single = (Element) xpath.selectSingleNode(elem);
        if (single != null) {
            report.getLog().debug("Powered - Custom");
            modified = this.getCustomPoweredBy(single);
        }

        return modified;
    }

    /**
     * Open the file, parses the HTML and the searches the defined tags to
     * replace with other content.
     *
     * @param directory
     *            Directory of the file.
     * @param filename
     *            Name of the file to modify.
     */
    protected void replace(final String/* ! */directory,
            final String/* ! */filename) {
        // assert filename != null && !filename.equals("") : "Invalid filename";
        report.getLog().info(
                "Processing file " + directory + File.separatorChar + filename);
        FileOutputStream output = null;
        Writer out = null;
        try {
            StringBuffer tot = new StringBuffer("<html>");
            BufferedReader input = new BufferedReader(new FileReader(directory
                    + File.separatorChar + filename));

            // Removes a line from the XML that is causing problems.
            String doctype = "";
            String namespace = "";
            String rline = input.readLine();
            while (rline != null) {
                if (!rline.contains("DOCTYPE")
                // TODO the next line should delete the namespace better
                        && !rline.trim().startsWith("<html")) {
                    rline = rline.trim();
                    tot.append(new StringBuffer(rline));
                } else if (rline.contains("DOCTYPE")) {
                    doctype = rline;
                } else if (rline.trim().startsWith("<html")) {
                    String line = rline.trim();
                    int pos = line.indexOf('>');
                    String rest = rline.substring(pos + 1);
                    tot.append(rest);
                    namespace = rline;
                }
                rline = input.readLine();
            }
            input.close();

            StringReader sr = new StringReader(tot.toString());

            SAXBuilder saxb = new SAXBuilder();
            saxb.setValidation(false);

            Element elem = saxb.build(sr).getRootElement();

            boolean modified = this.htmlReplacement(elem);

            // Writes the new html modified.
            if (modified) {
                report.getLog().debug("HTML re-written");
                XMLOutputter xmloutputter = new XMLOutputter(
                        Format.getRawFormat());
                output = new FileOutputStream(directory + File.separatorChar
                        + filename);
                out = new OutputStreamWriter(output);
                out.write(doctype);
                // TODO Add the namespace in the html tag.
                // TODO Keep the break lines for comments.
                // TODO The copyright sign is becoming corrupt.
                xmloutputter.output(elem, output);
            }
            sr.close();
        } catch (Throwable e) {
            report.getLog().error("General problem");
            report.getLog().debug(e);
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    report.getLog().error("IO Exception");
                    report.getLog().error(e);
                }
            }
            if (output != null) {
                try {
                    output.flush();
                    output.close();
                } catch (IOException e) {
                    report.getLog().error("IO Exception");
                    report.getLog().error(e);
                }
            }
        }
    }

}
